@@ -48,7 +48,7 @@ func (r *RoleType) UnmarshalJSON(data []byte) error {
 
 	*r = RoleType(str)
 
-	if !(*r).IsValid() {
+	if !r.IsValid() {
 		if (*r) != "" {
 			return fmt.Errorf("JSON validation error: invalid role value: %s", *r)
 		}
@@ -111,6 +111,7 @@ type OrgUserDTO struct {
 	OrgId         int64     `json:"orgId"`
 	UserId        int64     `json:"userId"`
 	Email         string    `json:"email"`
+	Name          string    `json:"name"`
 	AvatarUrl     string    `json:"avatarUrl"`
 	Login         string    `json:"login"`
 	Role          string    `json:"role"`
