@@ -3,12 +3,11 @@ import { QueryCtrl } from 'app/plugins/sdk';
 // import './css/query_editor.css';
 import TimegrainConverter from './time_grain_converter';
 import './editor/editor_component';
-import kbn from 'app/core/utils/kbn';
 
 import { TemplateSrv } from 'app/features/templating/template_srv';
 import { auto, IPromise } from 'angular';
-import { DataFrame } from '@grafana/data';
-import { PanelEvents } from '@grafana/data';
+import { DataFrame, PanelEvents, rangeUtil } from '@grafana/data';
+import { AzureQueryType, AzureMetricQuery } from './types';
 
 export interface ResultFormat {
   text: string;
@@ -20,34 +19,20 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
 
   defaultDropdownValue = 'select';
 
+  dummyDiminsionString = '+';
+
   target: {
+    // should be: AzureMonitorQuery
     refId: string;
-    queryType: string;
+    queryType: AzureQueryType;
     subscription: string;
-    azureMonitor: {
-      resourceGroup: string;
-      resourceName: string;
-      metricDefinition: string;
-      metricNamespace: string;
-      metricName: string;
-      dimensionFilter: string;
-      timeGrain: string;
-      timeGrainUnit: string;
-      allowedTimeGrainsMs: number[];
-      dimensions: any[];
-      dimension: any;
-      top: string;
-      aggregation: string;
-      aggOptions: string[];
-      timeGrains: Array<{ text: string; value: string }>;
-    };
+    azureMonitor: AzureMetricQuery;
     azureLogAnalytics: {
       query: string;
       resultFormat: string;
       workspace: string;
     };
     appInsights: {
-      rawQuery: boolean;
       // metric style query when rawQuery == false
       metricName: string;
       dimension: any;
@@ -63,12 +48,10 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
       timeGrain: string;
       timeGrains: Array<{ text: string; value: string }>;
       allowedTimeGrainsMs: number[];
-
-      // query style query when rawQuery == true
-      rawQueryString: string;
-      timeColumn: string;
-      valueColumn: string;
-      segmentColumn: string;
+    };
+    insightsAnalytics: {
+      query: any;
+      resultFormat: string;
     };
   };
 
@@ -106,12 +89,12 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     },
     appInsights: {
       metricName: this.defaultDropdownValue,
-      rawQuery: false,
-      rawQueryString: '',
-      dimension: 'none',
+      // dimension: [],
       timeGrain: 'auto',
-      timeColumn: 'timestamp',
-      valueColumn: '',
+    },
+    insightsAnalytics: {
+      query: '',
+      resultFormat: 'time_series',
     },
   };
 
@@ -137,9 +120,16 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
 
     this.migrateApplicationInsightsKeys();
 
+    this.migrateApplicationInsightsDimensions();
+
+    migrateMetricsDimensionFilters(this.target.azureMonitor);
+
     this.panelCtrl.events.on(PanelEvents.dataReceived, this.onDataReceived.bind(this), $scope);
     this.panelCtrl.events.on(PanelEvents.dataError, this.onDataError.bind(this), $scope);
-    this.resultFormats = [{ text: 'Time series', value: 'time_series' }, { text: 'Table', value: 'table' }];
+    this.resultFormats = [
+      { text: 'Time series', value: 'time_series' },
+      { text: 'Table', value: 'table' },
+    ];
     this.getSubscriptions();
     if (this.target.queryType === 'Azure Log Analytics') {
       this.getWorkspaces();
@@ -214,12 +204,13 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
       }
     }
 
+    const oldAzureTimeGrains = (this.target.azureMonitor as any).timeGrains;
     if (
-      this.target.azureMonitor.timeGrains &&
-      this.target.azureMonitor.timeGrains.length > 0 &&
+      oldAzureTimeGrains &&
+      oldAzureTimeGrains.length > 0 &&
       (!this.target.azureMonitor.allowedTimeGrainsMs || this.target.azureMonitor.allowedTimeGrainsMs.length === 0)
     ) {
-      this.target.azureMonitor.allowedTimeGrainsMs = this.convertTimeGrainsToMs(this.target.azureMonitor.timeGrains);
+      this.target.azureMonitor.allowedTimeGrainsMs = this.convertTimeGrainsToMs(oldAzureTimeGrains);
     }
 
     if (
@@ -269,6 +260,18 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     }
   }
 
+  migrateApplicationInsightsDimensions() {
+    const { appInsights } = this.target;
+
+    if (!appInsights.dimension) {
+      appInsights.dimension = [];
+    }
+
+    if (_.isString(appInsights.dimension)) {
+      appInsights.dimension = [appInsights.dimension as string];
+    }
+  }
+
   replace(variable: string) {
     return this.templateSrv.replace(variable, this.panelCtrl.panel.scopedVars);
   }
@@ -311,10 +314,8 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
       this.target.azureMonitor.resourceName = this.defaultDropdownValue;
       this.target.azureMonitor.metricName = this.defaultDropdownValue;
       this.target.azureMonitor.aggregation = '';
-      this.target.azureMonitor.timeGrains = [];
       this.target.azureMonitor.timeGrain = '';
-      this.target.azureMonitor.dimensions = [];
-      this.target.azureMonitor.dimension = '';
+      this.target.azureMonitor.dimensionFilters = [];
     }
   }
 
@@ -422,10 +423,8 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     this.target.azureMonitor.metricNamespace = this.defaultDropdownValue;
     this.target.azureMonitor.metricName = this.defaultDropdownValue;
     this.target.azureMonitor.aggregation = '';
-    this.target.azureMonitor.timeGrains = [];
     this.target.azureMonitor.timeGrain = '';
-    this.target.azureMonitor.dimensions = [];
-    this.target.azureMonitor.dimension = '';
+    this.target.azureMonitor.dimensionFilters = [];
     this.refresh();
   }
 
@@ -434,27 +433,22 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     this.target.azureMonitor.metricNamespace = this.defaultDropdownValue;
     this.target.azureMonitor.metricName = this.defaultDropdownValue;
     this.target.azureMonitor.aggregation = '';
-    this.target.azureMonitor.timeGrains = [];
     this.target.azureMonitor.timeGrain = '';
-    this.target.azureMonitor.dimensions = [];
-    this.target.azureMonitor.dimension = '';
+    this.target.azureMonitor.dimensionFilters = [];
   }
 
   onResourceNameChange() {
     this.target.azureMonitor.metricNamespace = this.defaultDropdownValue;
     this.target.azureMonitor.metricName = this.defaultDropdownValue;
     this.target.azureMonitor.aggregation = '';
-    this.target.azureMonitor.timeGrains = [];
     this.target.azureMonitor.timeGrain = '';
-    this.target.azureMonitor.dimensions = [];
-    this.target.azureMonitor.dimension = '';
+    this.target.azureMonitor.dimensionFilters = [];
     this.refresh();
   }
 
   onMetricNamespacesChange() {
     this.target.azureMonitor.metricName = this.defaultDropdownValue;
-    this.target.azureMonitor.dimensions = [];
-    this.target.azureMonitor.dimension = '';
+    this.target.azureMonitor.dimensionFilters = [];
   }
 
   onMetricNameChange(): IPromise<void> {
@@ -472,16 +466,18 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
         this.replace(this.target.azureMonitor.metricName)
       )
       .then((metadata: any) => {
-        this.target.azureMonitor.aggOptions = metadata.supportedAggTypes || [metadata.primaryAggType];
         this.target.azureMonitor.aggregation = metadata.primaryAggType;
-        this.target.azureMonitor.timeGrains = [{ text: 'auto', value: 'auto' }].concat(metadata.supportedTimeGrains);
         this.target.azureMonitor.timeGrain = 'auto';
-
         this.target.azureMonitor.allowedTimeGrainsMs = this.convertTimeGrainsToMs(metadata.supportedTimeGrains || []);
 
-        this.target.azureMonitor.dimensions = metadata.dimensions;
+        // HACK: this saves the last metadata values in the panel json ¯\_(ツ)_/¯
+        const hackState = this.target.azureMonitor as any;
+        hackState.aggOptions = metadata.supportedAggTypes || [metadata.primaryAggType];
+        hackState.timeGrains = [{ text: 'auto', value: 'auto' }].concat(metadata.supportedTimeGrains);
+        hackState.dimensions = metadata.dimensions;
+
         if (metadata.dimensions.length > 0) {
-          this.target.azureMonitor.dimension = metadata.dimensions[0].value;
+          //  this.target.azureMonitor.dimension = metadata.dimensions[0].value;
         }
 
         return this.refresh();
@@ -493,7 +489,7 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     const allowedTimeGrainsMs: number[] = [];
     timeGrains.forEach((tg: any) => {
       if (tg.value !== 'auto') {
-        allowedTimeGrainsMs.push(kbn.interval_to_ms(TimegrainConverter.createKbnUnitFromISO8601Duration(tg.value)));
+        allowedTimeGrainsMs.push(rangeUtil.intervalToMs(TimegrainConverter.createKbnUnitFromISO8601Duration(tg.value)));
       }
     });
     return allowedTimeGrainsMs;
@@ -502,7 +498,7 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
   generateAutoUnits(timeGrain: string, timeGrains: Array<{ value: string }>) {
     if (timeGrain === 'auto') {
       return TimegrainConverter.findClosestTimeGrain(
-        this.templateSrv.getBuiltInIntervalValue(),
+        '1m',
         _.map(timeGrains, o => TimegrainConverter.createKbnUnitFromISO8601Duration(o.value)) || [
           '1m',
           '5m',
@@ -520,13 +516,26 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
   }
 
   getAzureMonitorAutoInterval() {
-    return this.generateAutoUnits(this.target.azureMonitor.timeGrain, this.target.azureMonitor.timeGrains);
+    return this.generateAutoUnits(this.target.azureMonitor.timeGrain, (this.target.azureMonitor as any).timeGrains);
   }
 
   getApplicationInsightAutoInterval() {
     return this.generateAutoUnits(this.target.appInsights.timeGrain, this.target.appInsights.timeGrains);
   }
 
+  azureMonitorAddDimensionFilter() {
+    this.target.azureMonitor.dimensionFilters.push({
+      dimension: '',
+      operator: 'eq',
+      filter: '',
+    });
+  }
+
+  azureMonitorRemoveDimensionFilter(index: number) {
+    this.target.azureMonitor.dimensionFilters.splice(index, 1);
+    this.refresh();
+  }
+
   /* Azure Log Analytics */
 
   getWorkspaces = () => {
@@ -534,9 +543,18 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
       .getWorkspaces(this.target.subscription)
       .then((list: any[]) => {
         this.workspaces = list;
+
         if (list.length > 0 && !this.target.azureLogAnalytics.workspace) {
-          this.target.azureLogAnalytics.workspace = list[0].value;
+          if (this.datasource.azureLogAnalyticsDatasource.defaultOrFirstWorkspace) {
+            this.target.azureLogAnalytics.workspace = this.datasource.azureLogAnalyticsDatasource.defaultOrFirstWorkspace;
+          }
+
+          if (!this.target.azureLogAnalytics.workspace) {
+            this.target.azureLogAnalytics.workspace = list[0].value;
+          }
         }
+
+        return this.workspaces;
       })
       .catch(this.handleQueryCtrlError.bind(this));
   };
@@ -558,17 +576,7 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
   };
 
   get templateVariables() {
-    return this.templateSrv.variables.map(t => '$' + t.name);
-  }
-
-  /* Application Insights Section */
-
-  getAppInsightsAutoInterval() {
-    const interval = this.templateSrv.getBuiltInIntervalValue();
-    if (interval[interval.length - 1] === 's') {
-      return '1m';
-    }
-    return interval;
+    return this.templateSrv.getVariables().map(t => '$' + t.name);
   }
 
   getAppInsightsMetricNames() {
@@ -603,11 +611,11 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
       .catch(this.handleQueryCtrlError.bind(this));
   }
 
-  onAppInsightsQueryChange = (nextQuery: string) => {
-    this.target.appInsights.rawQueryString = nextQuery;
+  onInsightsAnalyticsQueryChange = (nextQuery: string) => {
+    this.target.insightsAnalytics.query = nextQuery;
   };
 
-  onAppInsightsQueryExecute = () => {
+  onQueryExecute = () => {
     return this.refresh();
   };
 
@@ -615,8 +623,27 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     return this.datasource.appInsightsDatasource.getQuerySchema().catch(this.handleQueryCtrlError.bind(this));
   };
 
+  removeGroupBy = (index: number) => {
+    const { appInsights } = this.target;
+    appInsights.dimension.splice(index, 1);
+    this.refresh();
+  };
+
   getAppInsightsGroupBySegments(query: any) {
-    return _.map(this.target.appInsights.dimensions, (option: string) => {
+    const { appInsights } = this.target;
+
+    // HACK alert... there must be a better way!
+    if (this.dummyDiminsionString && this.dummyDiminsionString.length && '+' !== this.dummyDiminsionString) {
+      if (!appInsights.dimension) {
+        appInsights.dimension = [];
+      }
+      appInsights.dimension.push(this.dummyDiminsionString);
+      this.dummyDiminsionString = '+';
+      this.refresh();
+    }
+
+    // Return the list of dimensions stored on the query object from the last request :(
+    return _.map(appInsights.dimensions, (option: string) => {
       return { text: option, value: option };
     });
   }
@@ -626,10 +653,6 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     this.refresh();
   }
 
-  toggleEditorMode() {
-    this.target.appInsights.rawQuery = !this.target.appInsights.rawQuery;
-  }
-
   updateTimeGrainType() {
     if (this.target.appInsights.timeGrainType === 'specific') {
       this.target.appInsights.timeGrainCount = '1';
@@ -654,3 +677,20 @@ export class AzureMonitorQueryCtrl extends QueryCtrl {
     this.refresh();
   }
 }
+
+// Modifies the actual query object
+export function migrateMetricsDimensionFilters(item: AzureMetricQuery) {
+  if (!item.dimensionFilters) {
+    item.dimensionFilters = [];
+  }
+  const oldDimension = (item as any).dimension;
+  if (oldDimension && oldDimension !== 'None') {
+    item.dimensionFilters.push({
+      dimension: oldDimension,
+      operator: 'eq',
+      filter: (item as any).dimensionFilter,
+    });
+    delete (item as any).dimension;
+    delete (item as any).dimensionFilter;
+  }
+}
